package org.apache.bcel.classfile;

import java.util.Arrays;
import java.io.DataOutputStream;
import org.apache.commons.lang3.ArrayUtils;
import java.io.IOException;
import java.io.DataInput;

public final class ModulePackages extends Attribute
{
    private int[] packageIndexTable;
    
    ModulePackages(final int nameIndex, final int length, final DataInput input, final ConstantPool constantPool) throws IOException {
        this(nameIndex, length, (int[])null, constantPool);
        final int packageCount = input.readUnsignedShort();
        this.packageIndexTable = new int[packageCount];
        for (int i = 0; i < packageCount; ++i) {
            this.packageIndexTable[i] = input.readUnsignedShort();
        }
    }
    
    public ModulePackages(final int nameIndex, final int length, final int[] packageIndexTable, final ConstantPool constantPool) {
        super((byte)23, nameIndex, length, constantPool);
        this.packageIndexTable = ((packageIndexTable != null) ? packageIndexTable : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    public ModulePackages(final ModulePackages c) {
        this(c.getNameIndex(), c.getLength(), c.getPackageIndexTable(), c.getConstantPool());
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitModulePackages(this);
    }
    
    @Override
    public Attribute copy(final ConstantPool constantPool) {
        final ModulePackages c = (ModulePackages)this.clone();
        if (this.packageIndexTable != null) {
            c.packageIndexTable = this.packageIndexTable.clone();
        }
        c.setConstantPool(constantPool);
        return c;
    }
    
    @Override
    public void dump(final DataOutputStream file) throws IOException {
        super.dump(file);
        file.writeShort(this.packageIndexTable.length);
        for (final int index : this.packageIndexTable) {
            file.writeShort(index);
        }
    }
    
    public int getNumberOfPackages() {
        return (this.packageIndexTable == null) ? 0 : this.packageIndexTable.length;
    }
    
    public int[] getPackageIndexTable() {
        return this.packageIndexTable;
    }
    
    public String[] getPackageNames() {
        final String[] names = new String[this.packageIndexTable.length];
        Arrays.setAll(names, i -> Utility.pathToPackage(super.getConstantPool().getConstantString(this.packageIndexTable[i], (byte)20)));
        return names;
    }
    
    public void setPackageIndexTable(final int[] packageIndexTable) {
        this.packageIndexTable = ((packageIndexTable != null) ? packageIndexTable : ArrayUtils.EMPTY_INT_ARRAY);
    }
    
    @Override
    public String toString() {
        final StringBuilder buf = new StringBuilder();
        buf.append("ModulePackages(");
        buf.append(this.packageIndexTable.length);
        buf.append("):\n");
        for (final int index : this.packageIndexTable) {
            final String packageName = super.getConstantPool().getConstantString(index, (byte)20);
            buf.append("  ").append(Utility.compactClassName(packageName, false)).append("\n");
        }
        return buf.substring();
    }
}
